@@ -2,9 +2,16 @@ from liquepy import trigger
 from liquepy import field
 from liquepy import element
 from liquepy import motion
-from liquepy import sra
+try:
+    from liquepy import sra
+except ImportError:
+    pass
 from liquepy import num
 from liquepy.functions import *
 from liquepy import fig
 from liquepy import __about__
-from liquepy import spatial
+try:
+    from liquepy import spatial
+except ImportError:
+    pass
+from . import esp
